﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Threading;

using LogicData;

namespace SilverlightSocketClient
{
    public partial class Page : UserControl
    {
        delegate void ScoreDataHandler(ScoreData scoreData);
        Teams _Teams = null;
        Socket socket = null;
        DnsEndPoint endPoint = null;

        //private static AutoResetEvent autoEvent = new AutoResetEvent(false);
        //private static AutoResetEvent[] autoSendReceiveEvents = new AutoResetEvent[]
        //{
        //    new AutoResetEvent(false),
        //    new AutoResetEvent(false)
        //};

        private const int Receive = 1;
        private const int Send = 0;

        public Page()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(Page_Loaded);
        }
        void Page_Loaded(object sender, RoutedEventArgs e)
        {
            //DnsEndPoint endPoint = new DnsEndPoint(Application.Current.Host.Source.DnsSafeHost, 4530);
            endPoint = new DnsEndPoint("localhost", 4530);
            socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            SocketAsyncEventArgs args = new SocketAsyncEventArgs();
            args.UserToken = socket;
            args.RemoteEndPoint = endPoint;
            args.Completed += new EventHandler<SocketAsyncEventArgs>(OnSocketConnectCompleted);
            socket.ConnectAsync(args);

            //autoEvent.WaitOne();
        }

        private void OnSocketConnectCompleted(object sender, SocketAsyncEventArgs e)
        {
            //autoEvent.Set();
            if (e.SocketError == SocketError.Success)
            {
                byte[] response = new byte[1024];
                e.SetBuffer(response, 0, response.Length);
                e.Completed -= new EventHandler<SocketAsyncEventArgs>(OnSocketConnectCompleted);
                e.Completed += new EventHandler<SocketAsyncEventArgs>(OnSocketReceive);
                Socket socket = (Socket)e.UserToken;
                socket.ReceiveAsync(e);

                LoginDataManager login = new LoginDataManager("dingpong", "1234");
                DataSend(login.GetXML().ToString());
            }
            else
            {
                Dispatcher.BeginInvoke(() => System.Windows.Browser.HtmlPage.Window.Alert(e.SocketError.ToString()));
            }
        }

        private void OnSend(object sender, SocketAsyncEventArgs e)
        {
            //autoSendReceiveEvents[Receive].Set();
        }

        internal String DataSend(string message)
        {
            if (socket.Connected)
            {
                Byte[] bytes = Encoding.UTF8.GetBytes(message);

                SocketAsyncEventArgs args = new SocketAsyncEventArgs();
                args.SetBuffer(bytes, 0, bytes.Length);
                args.UserToken = socket;
                args.RemoteEndPoint = endPoint;
                args.Completed += new EventHandler<SocketAsyncEventArgs>(OnSend);

                socket.SendAsync(args);

                //AutoResetEvent.WaitAll(autoSendReceiveEvents);

                return Encoding.UTF8.GetString(args.Buffer, args.Offset, args.BytesTransferred);
            }
            else
            {
                throw new SocketException((int)SocketError.NotConnected);
            }
        }

        private void OnSocketReceive(object sender, SocketAsyncEventArgs e)
        {
            //autoSendReceiveEvents[Send].Set();

            StringReader sr = null;
            try
            {
                string data = Encoding.UTF8.GetString(e.Buffer, e.Offset, e.BytesTransferred);
                sr = new StringReader(data);
                //Get initial team data
                if (_Teams == null && data.Contains("Teams"))
                {
                    XmlSerializer xs = new XmlSerializer(typeof(Teams));
                    _Teams = (Teams)xs.Deserialize(sr);
                    this.Dispatcher.BeginInvoke(UpdateBoard);
                }

                //Get updated score data
                if (data.Contains("ScoreData"))
                {
                    XmlSerializer xs = new XmlSerializer(typeof(ScoreData));
                    ScoreData scoreData = (ScoreData)xs.Deserialize(sr);
                    ScoreDataHandler handler = new ScoreDataHandler(UpdateScoreData);
                    this.Dispatcher.BeginInvoke(handler, new object[] { scoreData });
                }
            }
            catch { }
            finally
            {
                if (sr != null) sr.Close();
            }
            //Prepare to receive more data
            Socket socket = (Socket)e.UserToken;
            socket.ReceiveAsync(e);
        }

        private void UpdateBoard()
        {
            this.tbTeam1Score.Text = "0";
            this.tbTeam2Score.Text = "0";
            this.lbActions.Items.Clear();
            if (_Teams != null && _Teams.Team != null)
            {
                for (int i = 0; i < 2; i++)
                {
                    TeamsTeam team = _Teams.Team[i];
                    if (i == 0)
                    {
                        this.tbTeam1.Text = team.Name;
                    }
                    else
                    {
                        this.tbTeam2.Text = team.Name;
                    }
                }
            }
        }

        private void UpdateScoreData(ScoreData scoreData)
        {
            //Update Score
            this.tbTeam1Score.Text = scoreData.Team1Score.ToString();
            this.tbTeam2Score.Text = scoreData.Team2Score.ToString();
            //Update ball visibility
            if (scoreData.Action != ActionsEnum.Foul)
            {
                if (tbTeam1.Text == scoreData.TeamOnOffense)
                {
                    AnimateBall(this.BB1, this.BB2);
                }
                else //Team 2
                {
                    AnimateBall(this.BB2, this.BB1);
                }
            }
            if (this.lbActions.Items.Count > 11) this.lbActions.Items.Clear();
            this.lbActions.Items.Add(scoreData.LastAction);
            if (this.lbActions.Visibility == Visibility.Collapsed) this.lbActions.Visibility = Visibility.Visible;
        }

        private void AnimateBall(Image onBall, Image offBall)
        {
            this.FadeIn.Stop();
            Storyboard.SetTarget(this.FadeInAnimation, onBall);
            Storyboard.SetTarget(this.FadeOutAnimation, offBall);
            this.FadeIn.Begin();
        }

        public void Dispose()
        {
            //autoEvent.Close();
            //autoSendReceiveEvents[Send].Close();
            //autoSendReceiveEvents[Receive].Close();
            if (socket.Connected)
                socket.Close();
        }
    }
}
