﻿using System;
using System.Xml;
using System.Xml.Serialization;
using System.IO;

using GameLogic;

namespace LogicData
{
    public class LoginData
    {
        public string ID { get; set; }
        public string Password { get; set; }
    }

    public class LoginDataManager
    {
        /// <summary>
        /// 로그인 관련 데이터
        /// </summary>
        public LoginData loginData = new LoginData();

        public LoginDataManager(string id, string pw)
        {
            loginData.ID = id;
            loginData.Password = pw;
        }
        public LoginDataManager(string xml)
        {
            GetData(xml);
        }

        public string GetXML()
        {
            try
            {
                StringWriter sw = new StringWriter();
                XmlSerializer xm = new XmlSerializer(typeof(LoginData));
                xm.Serialize(sw, loginData);
                return sw.ToString();
            }
            catch (Exception e)
            {
                LogicManager.Instance.Util.LogError(e, "LoginData");
                return "";
            }
        }
        public LoginData GetData(string xml)
        {
            try
            {
                StringReader sr = new StringReader(xml);
                XmlSerializer xs = new XmlSerializer(typeof(LoginData));
                loginData = (LoginData)xs.Deserialize(sr);
                return loginData;
            }
            catch (Exception e)
            {
                LogicManager.Instance.Util.LogError(e, "LoginData");
                return loginData;
            }
        }
    }

    /// <summary>
    /// 회원 가입 관련 데이터
    /// </summary>
    public class RegisterData
    {
        public string ID { get; set; }
        public string EMailAddress { get; set; }
    }

    public class RegisterDataManager
    {
        public RegisterData RegisterData = new RegisterData();

        public RegisterDataManager(string id, string emailAddress)
        {
            RegisterData.ID = id;
            RegisterData.EMailAddress = emailAddress;
        }
        public RegisterDataManager(string xml)
        {
            GetData(xml);
        }

        public string GetXML()
        {
            try
            {
                StringWriter sw = new StringWriter();
                XmlSerializer xm = new XmlSerializer(typeof(RegisterData));
                xm.Serialize(sw, RegisterData);
                return sw.ToString();
            }
            catch (Exception e)
            {
                LogicManager.Instance.Util.LogError(e, "RegisterData");
                return "";
            }
        }
        public RegisterData GetData(string xml)
        {
            try
            {
                StringReader sr = new StringReader(xml);
                XmlSerializer xs = new XmlSerializer(typeof(RegisterData));
                RegisterData = (RegisterData)xs.Deserialize(sr);
                return RegisterData;
            }
            catch (Exception e)
            {
                LogicManager.Instance.Util.LogError(e, "RegisterData");
                return RegisterData;
            }
        }
    }



    /// <summary>
    /// 회원 가입 인증 데이터
    /// </summary>
    public class ActiveCodeData
    {
        public string ID { get; set; }
        public string ActiveCode { get; set; }
    }

    public class ActiveCodeDataManager
    {
        public ActiveCodeData ActiveCodeData = new ActiveCodeData();

        public ActiveCodeDataManager(string id, string activeCode)
        {
            ActiveCodeData.ID = id;
            ActiveCodeData.ActiveCode = activeCode;
        }
        public ActiveCodeDataManager(string xml)
        {
            GetData(xml);
        }

        public string GetXML()
        {
            try
            {
                StringWriter sw = new StringWriter();
                XmlSerializer xm = new XmlSerializer(typeof(ActiveCodeData));
                xm.Serialize(sw, ActiveCodeData);
                return sw.ToString();
            }
            catch (Exception e)
            {
                LogicManager.Instance.Util.LogError(e, "ActiveCodeData");
                return "";
            }
        }
        public ActiveCodeData GetData(string xml)
        {
            try
            {
                StringReader sr = new StringReader(xml);
                XmlSerializer xs = new XmlSerializer(typeof(ActiveCodeData));
                ActiveCodeData = (ActiveCodeData)xs.Deserialize(sr);
                return ActiveCodeData;
            }
            catch (Exception e)
            {
                LogicManager.Instance.Util.LogError(e, "ActiveCodeData");
                return ActiveCodeData;
            }
        }
    }


    /// <summary>
    /// 로그인 성공 관련 데이터
    /// </summary>
    public class LoginSuccessData
    {
        public string ID { get; set; }
    }

    public class LoginSuccessDataManager
    {
        public LoginSuccessData LoginSuccessData = new LoginSuccessData();

        public LoginSuccessDataManager(string id, int loginCount)
        {
            LoginSuccessData.ID = id;
        }
        public LoginSuccessDataManager(string xml)
        {
            GetData(xml);
        }
        public string GetXML()
        {
            try
            {
                StringWriter sw = new StringWriter();
                XmlSerializer xm = new XmlSerializer(typeof(LoginSuccessData));
                xm.Serialize(sw, LoginSuccessData);
                return sw.ToString();
            }
            catch (Exception e)
            {
                LogicManager.Instance.Util.LogError(e, "LoginSuccessData");
                return "";
            }
        }
        public LoginSuccessData GetData(string xml)
        {
            try
            {
                StringReader sr = new StringReader(xml);
                XmlSerializer xs = new XmlSerializer(typeof(LoginSuccessData));
                LoginSuccessData = (LoginSuccessData)xs.Deserialize(sr);
                return LoginSuccessData;
            }
            catch (Exception e)
            {
                LogicManager.Instance.Util.LogError(e, "LoginSuccessData");
                return LoginSuccessData;
            }
        }
    }
}