﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Timers;
using System.Xml;
using System.Reflection;
using System.Threading;
using System.Xml.Serialization;
using System.Collections;
using System.Data;
using System.Text;

using GameLogic;

// database manager 클레스 라이브러리
using DataBase;

using LogicData;

namespace Network
{
    class NetworkServer : INetwork
    {
        private TcpListener tcpl;
        private ClientManager manager;
        private Thread t;
        private event dString SendAllMsg;
        private event dStringIP SendGetMsgIP;
        private event dStringSelect SendToAllMsg, SendToOneMsg;

        public NetworkServer(INetworkServerMsg ui)
        {
            tcpl = new TcpListener(IPAddress.Any, ui.Port);
            manager = new ClientManager(GetMsgIP);
            SendAllMsg += new dString(manager.SendToAll);
            SendToAllMsg += new dStringSelect(manager.SendToAll);
            SendToOneMsg += new dStringSelect(manager.SendToOne);

            SendGetMsgIP += new dStringIP(ui.GetMsgIP);
            t = new Thread(Listen);

            IPHostEntry host = Dns.GetHostEntry(Dns.GetHostName());
            string msg = host.AddressList[0] + ":" + ui.Port;
            SendGetMsgIP(msg, ""); // startinfo
        }

        public void Start()
        {
            t.Start();

            // DB에 접속을 한다.
            DataBaseManager.Instance.DBConnect(DataBaseManager.DBType.MSSQL);
        }

        public void Stop()
        {
            t.Abort();
            if (tcpl != null) tcpl.Stop();
            if (manager != null) manager.Close();
        }

        private void Listen()
        {
            try
            {
                tcpl.Start();
                while (true)
                {
                    Socket s = tcpl.AcceptSocket();
                    Client c = manager.Add(s);

                    Console.WriteLine("접속 : " + c.MyIP);
                    Console.WriteLine("현재 접속자 숫자 : " + manager.TotalClientCount());
                }
            }
            catch (SocketException e)
            {
                SendGetMsgIP(e.Message, "");
                if (tcpl != null) tcpl.Stop();
            }
        }

        public void SendToAll(String d)
        {
            SendAllMsg(d);
        }

        public void SendToAll(String d, Client except)
        {
            SendToAllMsg(d, except);
        }
        public void SendToOne(String d, Client c)
        {
            SendToOneMsg(d, c);
        }

        public void GetMsgIP(String d, string IPFrom)
        {
            SendGetMsgIP(d, IPFrom);
        }
    }

    class ClientManager
    {
        private MessageQueue que;
        private event dStringIP SendToServerIP;

        public ClientManager(dStringIP sendToServerIP)
        {
            que = new MessageQueue();
            this.SendToServerIP = sendToServerIP;
        }

        public Client Add(Socket s)
        {
            Client c = new Client(s, que, Send);
            LogicManager.Instance.ClientList.Add(c.MyIP, c);
            c.Start();
            return c;
        }

        public int TotalClientCount()
        {
            return LogicManager.Instance.ClientList.Count;
        }

        public void Close()
        {
            SendToAll("서버가 종료되었습니다");
            foreach (Client c in LogicManager.Instance.ClientList.Values)
            c.Stop();
            LogicManager.Instance.ClientList.Clear();
        }

        public void SendToAll(String d)
        {
            foreach (Client c in LogicManager.Instance.ClientList.Values)
                c.Send(d);
        }

        public void SendToAll(String d, Client except)
        {
            foreach (Client c in LogicManager.Instance.ClientList.Values)
                if (c != except) c.Send(d);
        }

        public void SendToOne(String d, Client c)
        {
            c.Send(d);
        }

        private void Send(string IPFrom)
        {
            String d = que.Dequeue();

            SendToServerIP(d, IPFrom);
        }
    }
}
